@@ -11,8 +11,7 @@ from shutil import rmtree
 
 from bs4 import BeautifulSoup
 
-from dwca.rows import CoreRow
-from dwca.utils import _EmbeddedCSV
+from dwca.utils import _DataFile
 from dwca.descriptors import ArchiveDescriptor
 from dwca.exceptions import RowNotFound
 
@@ -84,8 +83,9 @@ class DwCAReader(object):
         #:
         self.source_metadata = None
 
-        self._corefile = _EmbeddedCSV(self.descriptor.core,
-                                      self._workin_directory_path)
+        self._corefile = _DataFile(self.descriptor.core,
+                                   self._workin_directory_path)
+        self._extensionfiles = [_DataFile(d, self._workin_directory_path) for d in self.descriptor.extensions]
 
     @property
     #TODO: decide, test and document what we guarantee about ordering
@@ -198,11 +198,14 @@ class DwCAReader(object):
         return self
 
     def next(self):
-        cl = self._corefile.get_row_by_index(self._corefile_pointer)
-        if cl:
+        r = self._corefile.get_row_by_position(self._corefile_pointer)
+        if r:
+            # Set up linked data so the CoreRow will know about them
+            r.link_extension_files(self._extensionfiles)
+            r.link_source_metadata(self.source_metadata)
+
             self._corefile_pointer = self._corefile_pointer + 1
-            return CoreRow(cl, self.descriptor, self._workin_directory_path,
-                           self.source_metadata)
+            return r
         else:
             raise StopIteration
 
